@@ -1220,9 +1220,45 @@ static void do_esc(struct tsm_vte *vte, uint32_t data)
 	}
 }
 
-static void csi_attribute(struct tsm_vte *vte)
+static void palette_rgb(struct tsm_vte *vte, int code, uint8_t *cr,
+			uint8_t *cg, uint8_t *cb)
+{
+	*cr = vte->palette[code][0];
+	*cg = vte->palette[code][1];
+	*cb = vte->palette[code][2];
+}
+
+static void cube_rgb(int code, uint8_t *cr, uint8_t *cg, uint8_t *cb)
 {
 	static const uint8_t bval[6] = { 0x00, 0x5f, 0x87, 0xaf, 0xd7, 0xff };
+	code -= 16;
+	*cb = bval[code % 6];
+	code /= 6;
+	*cg = bval[code % 6];
+	code /= 6;
+	*cr = bval[code % 6];
+}
+
+static void greyscale_rgb(int code, uint8_t *cr, uint8_t *cg, uint8_t *cb)
+{
+	uint8_t lightness = (code - 232) * 10 + 8;
+	*cr = *cg = *cb = lightness;
+}
+
+static void lookup_color(struct tsm_vte *vte, int color, uint8_t *cr,
+			 uint8_t *cg, uint8_t *cb)
+{
+	if (color < 16) {
+		palette_rgb(vte, color, cr, cg, cb);
+	} else if (color < 232) {
+		cube_rgb(color, cr, cg, cb);
+	} else {
+		greyscale_rgb(color, cr, cg, cb);
+	}
+}
+
+static void csi_attribute(struct tsm_vte *vte)
+{
 	unsigned int i, code, val;
 	uint8_t cr, cg, cb;
 
@@ -1390,19 +1426,8 @@ static void csi_attribute(struct tsm_vte *vte)
 				if (code < 16) {
 					//no change
 					cb = cg = cr = 0;
-				} else if (code < 232) {
-					code -= 16;
-					cb = bval[code % 6];
-					code /= 6;
-					cg = bval[code % 6];
-					code /= 6;
-					cr = bval[code % 6];
-					code = -1;
 				} else {
-					code = (code - 232) * 10 + 8;
-					cr = code;
-					cg = code;
-					cb = code;
+					lookup_color(vte, code, &cr, &cg, &cb);
 					code = -1;
 				}
 				i += 2;
@@ -2059,12 +2084,97 @@ static void do_osc_collect(struct tsm_vte *vte, uint32_t val) {
 	vte->osc_len += len;
 }
 
-static void do_osc_end(struct tsm_vte *vte) {
+static void vte_write_xcolor(struct tsm_vte *vte, char *code,
+			     const char *end_seq,
+			     uint8_t r, uint8_t g, uint8_t b)
+{
+	char buf[32];
+	snprintf(buf, sizeof(buf), "\e]%s;rgb:%02x%02x/%02x%02x/%02x%02x%s",
+		 code, r, r, g, g, b, b, end_seq);
+	vte_write(vte, buf, strlen(buf));
+}
+
+static void do_osc_4(struct tsm_vte *vte, const char *data, const char *end_seq)
+{
+	char buf[32];
+	uint8_t cr, cg, cb;
+	const char *c = data;
+
+	while (*c) {
+		// parse color code
+		unsigned int code = 0;
+		while (*c >= '0' && *c <= '9') {
+			code = code * 10 + (*c++ - '0');
+		}
+		if (*c++ != ';') {
+			break;
+		}
+
+		// parse value field
+		if (*c == '?') {
+			c++;
+			// handle rgb value query
+			snprintf(buf, sizeof(buf), "4;%u", code);
+			lookup_color(vte, code, &cr, &cg, &cb);
+			vte_write_xcolor(vte, buf, end_seq, cr, cg, cb);
+		} else {
+			// parsing of new rgb value to assign to code would go
+			// here. For now, skip past field contents.
+			while (*c && *c != ';') {
+				c++;
+			}
+		}
+		if (*c++ != ';') {
+			break;
+		}
+	}
+}
+
+static bool do_osc_internal(struct tsm_vte *vte, const char *end_seq)
+{
+	// OSC 4, 10, and 11 query or change the RGB values of color numbers in
+	// the foreground/background (SGR 38 & 48), 3-bit (SGR 30-37 & 40-47),
+	// 4-bit (SGR 90-97 & 100-107), and 256-color (SGR 38;5 & 48;5) indexed
+	// color tables. We don't support changing the tables, but we do support
+	// querying them.
+	if (!strncmp(vte->osc_arg, "4;", 2)) {
+		do_osc_4(vte, vte->osc_arg + 2, end_seq);
+		return true;
+	}
+	if (!strncmp(vte->osc_arg, "10;?", 4)) {
+		vte_write_xcolor(vte, "10", end_seq,
+				 vte->def_attr.fr, vte->def_attr.fg,
+				 vte->def_attr.fb);
+		return true;
+	}
+	if (!strncmp(vte->osc_arg, "11;?", 4)) {
+		vte_write_xcolor(vte, "11", end_seq,
+				 vte->def_attr.br, vte->def_attr.bg,
+				 vte->def_attr.bb);
+		return true;
+	}
+	return false;
+}
+
+/* XTerm OSC responses use the same sequence terminator used in the query */
+static const char *osc_end_seq(uint32_t end)
+{
+	if (end == 0x07) {
+		return "\x07";
+	}
+	return "\x1b\x5c";
+}
+
+static void do_osc_end(struct tsm_vte *vte, uint32_t val) {
+	const char *end_seq = osc_end_seq(val);
+	vte->osc_arg[vte->osc_len] = 0;
+
+	if (do_osc_internal(vte, end_seq)) {
+		return;
+	}
 	if (!vte->osc_cb) {
 		return;
 	}
-
-	vte->osc_arg[vte->osc_len] = 0;
 	vte->osc_cb(vte, vte->osc_arg, vte->osc_len, vte->osc_data);
 }
 
@@ -2115,7 +2225,7 @@ static void do_action(struct tsm_vte *vte, uint32_t data, int action)
 			do_osc_collect(vte, data);
 			break;
 		case ACTION_OSC_END:
-			do_osc_end(vte);
+			do_osc_end(vte, data);
 			break;
 		default:
 			llog_warning(vte, "invalid action %d", action);
